// Copyright (c) 2023 Dominic Masters
// 
// This software is released under the MIT License.
// https://opensource.org/licenses/MIT

#include "CharacterController2D.hpp"

using namespace Dawn;

CharacterController2D::CharacterController2D(SceneItem *i) :
  SceneItemComponent(i)
{
}

void CharacterController2D::onStart() {
  useEvent([&](float_t delta){
    // Friction
    velocity -= velocity * friction * delta;

    auto myCollider = item->getComponent<Collider2D>();
    glm::vec2 currentPosition(
      this->transform->getLocalPosition().x,
      this->transform->getLocalPosition().z
    );


    // Perform movement and check for collisions.
    glm::vec2 moveAmount;
    if(myCollider == nullptr) {
      moveAmount = velocity;
    } else {
      // Perform sweep
      auto allColliders = getScene()->findComponents<Collider2D>();
      auto itColliders = allColliders.begin();

      struct CharacterController2DCollisionEventInfo info;
      bool_t result = false;

      // Check for collisions, definitely not working 100% yet 
      while(itColliders != allColliders.end()) {
        auto c = *itColliders;
        ++itColliders;
        if(c == myCollider) continue;
        result = myCollider->getCollidingResult(
          velocity,
          c,
          info.normal,
          info.entryTime,
          info.exitTime,
          info.entryPoint,
          info.exitPoint
        ) && info.entryTime <= delta;
        if(result) {
          info.collider = c;
          break;
        }
      }
      
      if(result) {
        moveAmount = glm::vec2(0, 0);
        velocity = glm::vec2(0, 0);
        this->eventCollision.invoke(info);
      } else {
        moveAmount = velocity;
      }
    }

    if(
      mathAbs<float_t>(moveAmount.x) <= 0.001f &&
      mathAbs<float_t>(moveAmount.y) <= 0.001f
    ) return;

    transform->setLocalPosition(
      transform->getLocalPosition() + (glm::vec3(moveAmount.x, 0, moveAmount.y) * delta)
    );
  }, getScene()->eventSceneUpdate);
}